function love.load()
	myList = {
		"alpha", "beta", "gamma", "delta", "epsilon", "zeta", "eta", "theta", "iota", "kappa", "lambda", "mu",
		"nu", "xi", "omicron", "pi", "rho", "sigma", "tau", "upsilon", "phi", "chi", "psi", "omega"
	}
	
	listHeight = 300
	itemHeight = 60
	listScroll = 0
	--			scrolling, mouseY
	scrolling = {false, false}
	font = love.graphics.newFont(24)
	love.graphics.setFont(font)
	
	scrollSide = {false, 1, 2}
	progressiveScroll = {false, 1, {}}
	
	totalHeight = itemHeight*#myList-listHeight
	deadZone = 5
	
	selection = 0
	love.window.setMode(250, 350)
	love.window.setTitle("Scroller")
	
	love.graphics.setBackgroundColor(235, 245, 255)
end

function love.update(dt)
	if scrollSide[1] and not scrolling[1] then
		scrollSide[1] = math.max(0, scrollSide[1]-dt)
		if scrollSide[1] == 0 then
			scrollSide[1] = false
		end
	end
	
	if scrolling[1] then
		local dif = scrolling[2]-love.mouse.getY()
		
		local fact = scrolling[1] + dif/totalHeight
		if fact <= 0 then
			fact = 0
		elseif fact >= 1 then
			fact = 1
		end
		listScroll = fact
		
		local lim = 3
		while #progressiveScroll[3] <= lim do
			table.insert(progressiveScroll[3], love.mouse.getY())
		end
		table.remove(progressiveScroll[3], 1)
	elseif scrolling[2] then
		if math.abs(love.mouse.getY()-scrolling[2]) >= deadZone then
			scrolling[1] = listScroll
			scrollSide[1] = scrollSide[3]
		end
	else
		if progressiveScroll[1] then
			progressiveScroll[1] = math.max(0, progressiveScroll[1]-dt)
			
			local f = progressiveScroll[1]/progressiveScroll[2]
			local add = progressiveScroll[3][1]-progressiveScroll[3][ #progressiveScroll[3] ]
			
			local speed = .5
			add = add*f*speed
			
			listScroll = listScroll + add/totalHeight
			if listScroll <= 0 then
				listScroll = 0
			elseif listScroll >= 1 then
				listScroll = 1
			end
			
			if progressiveScroll[1] == 0 then
				progressiveScroll[1] = false
				progressiveScroll[3] = {}
			end
		end
	end
end

function love.draw()
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setScissor(25, 25, 200, 300)
	love.graphics.setLineWidth(2)
	
	local i1 = math.floor(totalHeight*listScroll/itemHeight)
	local off = math.mod(totalHeight*listScroll/itemHeight, 1)
	local i2 = math.ceil((totalHeight*listScroll+listHeight)/itemHeight)
	i1 = math.max(1, i1+1)
	i2 = math.min(#myList, i2)
	
	print("i1 = " .. i1 .. ", i2 = " .. i2)
	for i = i1, i2 do
		local py = 25+(itemHeight*(1-off))+(i-i1-1)*itemHeight
		love.graphics.setColor(155, 155, 155, 255)
		if selection == i then
			love.graphics.setColor(205, 205, 155, 255)
		end
		
		love.graphics.rectangle("fill", 25, py, 250, itemHeight)
		love.graphics.setColor(105, 115, 135, 255)
		love.graphics.rectangle("line", 25, py, 250, itemHeight)
		
		local lm = 0
		if selection == i then
			lm = 2
			love.graphics.setColor(255, 255, 255, 255)
		end
		
		for a = -lm, lm+1 do
			local offx = a
			if a == lm+1 then
				love.graphics.setColor(105, 115, 135, 255)
				offx = 0
			end
			love.graphics.print(myList[i], 25+10 + offx, math.floor(py+itemHeight/2-font:getHeight()/2))
		end
	end
	
	love.graphics.setScissor()
	love.graphics.setColor(155, 135, 115, 255)
	love.graphics.rectangle("line", 25, 25, 200, 300)
	
	if scrollSide[1] then
		love.graphics.setColor(55, 55, 55, 255*(math.min(1, scrollSide[1]/scrollSide[2])))
		local barSize = math.max(20, listHeight/#myList)
		
		local pos = (listHeight-barSize)*listScroll
		love.graphics.rectangle("fill", 230, 25+pos, 10, barSize, 5, 5, 32)
		love.graphics.setLineWidth(1)
		love.graphics.rectangle("line", 230, 25+pos, 10, barSize, 5, 5, 32)
	end
end

function love.keypressed(key)
	if key == "escape" then
		if selection > 0 then
			selection = 0
			love.window.setTitle("Scroller")
		else
			love.event.quit()
		end
	end
end

function love.mousepressed(x, y, button)
	if button == 1 and inside(x, y, 1, 1, 25, 25, 200, 300) then
		scrolling[2] = y
		progressiveScroll[1] = false
		progressiveScroll[3] = {}
	end
end

function love.mousereleased(x, y, button)
	if button == 1 then
		if not scrolling[1] then
			if inside(x, y, 1, 1, 25, 25, 200, 300) then
				local py = y-25+listScroll*totalHeight
				py = math.floor(py/itemHeight)+1
				if py >= 1 and py <= #myList then
					selection = py
					love.window.setTitle(myList[py])
				end
			end
		end
		scrolling = {false, false}
		if progressiveScroll[3][1] and math.abs(progressiveScroll[3][1]-progressiveScroll[3][ #progressiveScroll[3] ]) >= deadZone then
			progressiveScroll[1] = progressiveScroll[2]
		end
	end
end

function love.wheelmoved(x, y)
	if not scrolling[1] and not scrolling[2] then
		listScroll = listScroll - y*(itemHeight/5/totalHeight)
		if listScroll <= 0 then
			listScroll = 0
		elseif listScroll >= 1 then
			listScroll = 1
		end
		scrollSide[1] = scrollSide[3]
	end
end

function inside(x1, y1, w1, h1, x2, y2, w2, h2)
	return x1 >= x2 and y1 >= y2 and x1+w1 <= x2+w2 and y1+h1 <= y2+h2
end